#ifndef BUFFERING_H
#define BUFFERING_H

#include <QObject>
#include <array>
#include "voltagedata.h"
// We use QObject because we need to manually connect its slots with others.
// Warning: setCount is not Thread-safe!! Normally we won't use it.
// Warning: No template is used because qt doesn't support template in qobject.

#define portnum 8
#define bufsize 5

class Buffering : public QObject
{
    Q_OBJECT
public:
    explicit Buffering(QObject *parent = 0);
signals:
    void packGathered(std::array<volt_data, 8>*);
public slots:
    void setCount(unsigned int);
    void writeBuffer(const volt_data*);
private:
    std::array<std::array<bool, 8>, bufsize> flags; // Probably could be better.
    std::array<std::array<volt_data, 8>, bufsize> data;
    typename volt_data::tIDX cnt;
};

#define WARN_TOO_PREVIOUS 1
#define WARN_TOO_AHEAD 2
#define WARN_DISCARDED 3

class BufferingWithWarning : public QObject
{
    Q_OBJECT
public:
    explicit BufferingWithWarning(QObject *parent = 0);
signals:
    void packGathered(std::array<volt_data, 8>*);
    void warn(int);
public slots:
    void setCount(unsigned int);
    void writeBuffer(const volt_data*);
private:
    std::array<std::array<bool, 8>, bufsize> flags; // Probably could be better.
    std::array<std::array<volt_data, 8>, bufsize> data;
    typename volt_data::tIDX cnt;
};

#endif // BUFFERING_H
