#include "buffering.h"

Buffering::Buffering(unsigned int vol_thres, unsigned int idx_thres, QObject *parent) :
    isstarted(false),
    m_vol_thres(vol_thres),
    m_idx_thres(idx_thres),
    data(NULL),
    QObject(parent)
{}

inline bool allabove(unsigned int s1, const std::array<unsigned int, 8>& s2)
{
    return (s2[0]>s1)&&(s2[1]>s1)&&(s2[2]>s1)&&(s2[3]>s1)&&(s2[4]>s1)&&(s2[5]>s1)&&(s2[6]>s1)&&(s2[7]>s1);
}

void Buffering::write(const volt_data * s)
{
    if(!isstarted)
    {
        if(s->voltage>=m_vol_thres)
        {
            isstarted = true;
            sindex = s->index;

            data = new std::array<std::vector<unsigned int>, 8>();
            threscnt.fill(0);
            for(int i = 0; i < 8; i++)
            {
                data->at(i).reserve(10240);
            }
            emit started(true);
        }
    }
    if(isstarted)
    {
        if(s->index < sindex)
        {
            warn(WARN_TOO_PREVIOUS);
            return;
        }
        unsigned int l = s->index - sindex;
        unsigned int pport = s->port;
        if(l >= data->at(pport).size())
        {
            if(l - data->at(pport).size() > 10000)
            {
                warn(WARN_TOO_AHEAD);
                return;
            }else
            {
                data->at(pport).resize(l+1, 1024); // 1024 means the data is invalid
            }
        }
        data->at(pport)[l] = s->voltage;
        if(s->voltage<m_vol_thres)
        {
            threscnt[pport]++;
            if(allabove(m_idx_thres, threscnt))
            {
                isstarted = false;
                emit stopped(data, true); // When stopped(..) is emitted, the object no longer consider *data as part of its control.
                return;
            }
        }else
        {
            threscnt[pport] = 0;
        }
    }
}

void Buffering::stop()
{
    isstarted = false;
    emit stopped(data, false);
}
