#include "dataentry.h"
#include <QVector>
#include <QDataStream>
#include <QFile>
DataEntry::DataEntry(const QString& fp, const QString& mdlfp)
    : mfp(fp), mmdlfp(mdlfp), mdl(NULL)
{
    QFile* fi = new QFile(fp);
    if(!fi->exists()) { fi->open(QIODevice::WriteOnly); fi->close(); return; }
    fi->open(QIODevice::ReadOnly);
    QDataStream mds;
    mds.setDevice(fi);

    QVector<unsigned int> utmp;
    QVector<QVector<double> > ftmp;

    mds >> utmp;
    mds >> ftmp;

    readSVMModel(mdlfp);
    ulist = utmp.toStdVector();
    for(unsigned int i = 0; i < ftmp.size(); i++)
    {
        flist.push_back(ftmp[i].toStdVector());
    }
    fi->close();
}

DataEntry::~DataEntry()
{
    QFile* fi = new QFile(mfp);
    fi->open(QIODevice::WriteOnly);

    QDataStream mds;
    mds.setDevice(fi);
    mds << QVector<unsigned int>::fromStdVector(ulist);

    QVector<QVector<double> > tmp;
    for(unsigned int i = 0; i < flist.size(); i++)
    {
        tmp.push_back(QVector<double>::fromStdVector(flist[i]));
    }

    mds << tmp;

    writeSVMModel(mmdlfp);

    fi->close();
}

void DataEntry::regFeatureVector(unsigned int id, const std::vector<double> & s)
{
    ulist.push_back(id);
    flist.push_back(s);
}

const std::vector<std::vector<double> >& DataEntry::getAllFeatureVectors()
{
    return flist;
}

const std::vector<unsigned int>& DataEntry::getUserList()
{
    return ulist;
}

svm_model* DataEntry::getSVMModel()
{
    return svm_load_model(mmdlfp.toLatin1());
}

void DataEntry::regSVMModel(const svm_model* src)
{
    svm_save_model(mmdlfp.toLatin1(), src);
}

void DataEntry::readSVMModel(const QString& src)
{
    mdl = svm_load_model(src.toLatin1());
}

void DataEntry::writeSVMModel(const QString& dest)
{
    if(mdl!=NULL)
    {
        svm_save_model(dest.toLatin1(), mdl);
    }
}
