#ifndef DATAENTRY_H
#define DATAENTRY_H

#include <vector>
#include "svm.h"
#include <QFile>
#include <QDataStream>
class DataEntry
{
public:
    DataEntry(const QString& fp, const QString& mdlfp);
    ~DataEntry();
    // Data Entry is not responsible for type transformation. It has to be done elsewhere.
    // Data Entry will model the time it is created, but it won't load feature vector (for efficiency). The feature vector and its corresponding user will be stored elsewhere.
    void regFeatureVector(unsigned int id, const std::vector<double>&);
    std::vector<double> getFeatureVector(unsigned int id); // If all the feature vectors are in memory, the specified vector will be copied from memory, otherwise it will be loaded from disc.
    const std::vector<std::vector<double> >& getAllFeatureVectors();
    const std::vector<unsigned int>& getUserList();

    // Feature Vector Part End.
    svm_model* getSVMModel(); // NULL means not ready.
    void regSVMModel(const svm_model*);

    static svm_parameter getDefaultParam()
    {
        svm_parameter tmp;
        tmp.svm_type = C_SVC;
        tmp.kernel_type = RBF;
        tmp.degree = 3;
        tmp.gamma = 0;	// 1/num_features
        tmp.coef0 = 0;
        tmp.nu = 0.5;
        tmp.cache_size = 100;
        tmp.C = 1;
        tmp.eps = 1e-3;
        tmp.p = 0.1;
        tmp.shrinking = 1;
        tmp.probability = 0;
        tmp.nr_weight = 0;
        tmp.weight_label = NULL;
        tmp.weight = NULL;
        return tmp;
    }
private:
    QString mfp;
    void readSVMModel(const QString& src);
    QString mmdlfp;
    void writeSVMModel(const QString& dest);
    std::vector<unsigned int> ulist;
    std::vector<std::vector<double> > flist;
    svm_model* mdl;
};

#endif
