#include "gridcrossvalidating.h"
#include "svm.h"
#include <cassert>
GridCrossValidating::GridCrossValidating(DataEntry& de, QObject *parent) :
    mde(de), QObject(parent)
{
}

void constructProblem2(const std::vector<unsigned int>& ids, const std::vector<std::vector<double> >& features, svm_problem* res)
{
    assert(ids.size()==features.size());
    res->l = ids.size();
    res->y = new double[res->l];
    res->x = new svm_node*[res->l];
    unsigned int j;
    for(unsigned i = 0; i < res->l; i++)
    {
        res->y[i] = ids[i];
        res->x[i] = new svm_node[features[i].size()+1];
        for(j = 0; j < features[i].size(); j++)
        {
            res->x[i][j].index = j;
            res->x[i][j].value = features[i][j];
        }
        res->x[i][features[i].size()].index = -1;
    }
}

void deleteProblem2(svm_problem* s)
{
    for(unsigned int i = 0; i < s->l; i++)
    {
        delete [] s->x[i];
    }
    delete [] s->y;
}

void GridCrossValidating::validate()
{
    const std::vector<std::vector<double> >& vecs = mde.getAllFeatureVectors(); // which includes the added vector;
    const std::vector<unsigned int>& ids = mde.getUserList();

    svm_problem* prob = new svm_problem;
    constructProblem2(ids, vecs, prob);

    svm_parameter def = DataEntry::getDefaultParam();

    double *target = new double[prob->l];
    double i, j;
    unsigned int k;

    double bC, bgamma;
    unsigned int ttc = 0;
    unsigned int rec = 0;
    for(i = -5; i <= 15; i+=2)
    {
        for(j = -15; j <= 3; j+=2)
        {
            ttc = 0;
            def.C = pow(2, i);
            def.gamma = pow(2, j);
            svm_cross_validation(prob, &def, 3, target); // n_fold may change;
            for(k = 0; k < prob->l; k++)
            {
                if(target[k] == prob->y[k])
                    ++ttc;
            }
            if(ttc > rec) {
                rec = ttc;
                bC = def.C;
                bgamma = def.gamma;
            }
        }
    }
    def.C = bC;
    def.gamma = bgamma;
    svm_model* trained = svm_train(prob, &def);

    mde.regSVMModel(trained);
    delete target;
    emit validationComplete(svm_get_nr_class(trained));
}
