#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "buffering.h"
#include "voltagedatacollecting.h"
#include "dimensionreducing.h"
#include "badsamplefixing.h"

#include <armadillo>
#include "fourierfeatureextracting.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    valid_packets(0),
    discarded_packets(0),
    invalid_packets(0),
    ui(new Ui::MainWindow)
{
    m_b = new Buffering(338, 10, this);

    ui->setupUi(this);
    info = QSerialPortInfo::availablePorts();
    for(const auto& i : info)
    {
        ui->cbPorts->addItem(i.manufacturer() + i.portName());
    }

    m_vdc = new VoltageDataCollectingWithWarning(info[0], this);

    mde = new DataEntry("data.ety", "mdl.ety");
    mr = new Registering(*mde, this);
    mt = new Testing(*mde, this);
    mgcv = new GridCrossValidating(*mde, this);
    ui->btnStop->setEnabled(false);
    // Connection Inside MainWindow
    connect(ui->btnListen, SIGNAL(clicked()), this, SLOT(goListen()));
    connect(ui->btnStop, SIGNAL(clicked()), this, SLOT(stopListen()));

    connect(ui->btnCrossValidate, SIGNAL(clicked()), mgcv, SLOT(validate()));
    connect(mgcv, SIGNAL(validationComplete(int)), this, SLOT(valCompl(int)));

    connect(ui->rdNone, SIGNAL(clicked()), this, SLOT(chg2nothmod()));
    connect(ui->rdRecognize, SIGNAL(clicked()), this, SLOT(chg2recogmod()));
    connect(ui->rdRegister, SIGNAL(clicked()), this, SLOT(chg2regmod()));

    connect(ui->txtMsg, SIGNAL(textChanged()), this, SLOT(autoScroll()));
    connect(ui->btnClear, SIGNAL(clicked()), this, SLOT(clearMsg()));
    // Connection To MainWindow
    connect(m_vdc, SIGNAL(packetReady(const volt_data*)), this, SLOT(updatePacketStatus(const volt_data*)));
    connect(m_b, SIGNAL(stopped(std::array<std::vector<uint>,8>*,bool)), this, SLOT(evaluateData(std::array<std::vector<uint>,8>*)));
    connect(m_b, SIGNAL(started(bool)), this, SLOT(bufstarted()));
    connect(m_b, SIGNAL(warn(int)), this, SLOT(bufwarn(int)));
    connect(m_vdc, SIGNAL(warn(int)), this, SLOT(packetwarn(int)));
    connect(m_vdc, SIGNAL(error(int)), this, SLOT(packeterr(int)));

    connect(mr, SIGNAL(registerComplete()), this, SLOT(regCompl()));
    connect(mt, SIGNAL(result(uint)), this, SLOT(tstCompl(uint)));
    //connect(mt, SIGNAL(probResult(double*,int)), this, SLOT(showProbs(double*,int)));
    // Internal Connection
    connect(m_vdc, SIGNAL(packetReady(const volt_data*)), m_b, SLOT(write(const volt_data*)));

    connect(m_vdc, SIGNAL(dCur(long)), this, SLOT(showCurThread(long)));


    pvec.push_back(ui->lV0);
    pvec.push_back(ui->lV1);
    pvec.push_back(ui->lV2);
    pvec.push_back(ui->lV3);
    pvec.push_back(ui->lV4);
    pvec.push_back(ui->lV5);
    pvec.push_back(ui->lV6);
    pvec.push_back(ui->lV7);

/*    ivec.push_back(ui->lbli0);
    ivec.push_back(ui->lbli1);
    ivec.push_back(ui->lbli2);
    ivec.push_back(ui->lbli3);
    ivec.push_back(ui->lbli4);
    ivec.push_back(ui->lbli5);
    ivec.push_back(ui->lbli6);
    ivec.push_back(ui->lbli7);*/

    chg2nothmod();


}

void MainWindow::showCurThread(long s)
{
//    showMsg(QString::number(s));
}

MainWindow::~MainWindow()
{
    delete mde;
    delete ui;
}

void MainWindow::bufstarted()
{
    showMsg(tr("Recording Started."));
}

void MainWindow::goListen()
{
    //m_vdc = new VoltageDataCollectingWithWarning(info[ui->cbPorts->currentIndex()], this);
    m_vdc->start();
    ui->btnListen->setEnabled(false);
    ui->btnStop->setEnabled(true);
    if(m_vdc->getSerialPort()->isOpen())
        showMsg(tr("Listening Started."));
}

void MainWindow::stopListen()
{
    ui->btnStop->setEnabled(false);
    ui->btnListen->setEnabled(true);
    if(m_vdc->getSerialPort()->isOpen())
        showMsg(tr("Listening Stopped."));
    m_vdc->stop();
}

void MainWindow::setCnt()
{
    // This Slot is useless now.
}

inline double voltage_transform(unsigned int vol)
{
    return (double)vol / 1024.0 * 5.0;
}

void MainWindow::updatePacketStatus(const volt_data *s)
{
    curcnt = s->index;
    updateCnt();
    valid_packets++;
    updatePacketStat();
    pvec[s->port]->display(voltage_transform(s->voltage));
    //ivec[s->port]->display(s->index));
    if(s->voltage>=410)
    {
        showMsg(QString::number(s->port));
    }
}

void MainWindow::updatePacketStat()
{
    ui->lblVPacket->setText(QString::number(valid_packets));
    ui->lblDPacket->setText(QString::number(discarded_packets));
    ui->lblIPacket->setText(QString::number(invalid_packets));
    //ui->lblPacket->setText(tr("Valid Packets: ") + QString::number(valid_packets) + " Corrupted Packets: " + QString::number(discarded_packets) + " Invalid Packets: " + QString::number(invalid_packets));
}

void MainWindow::updateCnt()
{
    ui->lblCCount->setText(QString::number(curcnt));
}

void MainWindow::bufwarn(int s)
{
/*    switch (s) {
    case WARN_TOO_AHEAD:
//        showMsg(" >>");
        break;
    case WARN_TOO_PREVIOUS:
//        showMsg(" <<");
        break;
    }*/
}

void MainWindow::packetwarn(int s)
{
    discarded_packets++;
    if(s==WARN_INVALID_PACKET)
    {
        invalid_packets++;
    }
//    showMsg(" C");
}

void MainWindow::packeterr(int s)
{
    switch (s) {
    case ERR_NEED_PERMISSION:
        showMsg(tr("Need Permission."));
        break;
    case ERR_ALREADY_OPENED:
        showMsg(tr("Port Already Opened."));
        break;
    case ERR_NOT_OPENED:
        showMsg(tr("Port Not Opened."));
        break;
    case ERR_UNKNOWN:
        showMsg(tr("Unknown Error"));
        break;
    }
}

void MainWindow::showMsg(QString src)
{
    ui->txtMsg->setHtml(ui->txtMsg->toHtml() + src);
}

void MainWindow::evaluateData(std::array<std::vector<unsigned int>, 8>* s)
{
    showMsg(tr("Data Gathered."));
    unsigned int loss = 0;
    unsigned int i;
    /*
    for(i = 0; i < 8; i++)
    {
        loss = 0;
        for(const auto& j: s->at(i))
        {
            if(j==1024) loss++;
        }
        showMsg(QString("Loss.") + QString::number(loss) + " Total:" + QString::number(s->at(i).size()) + "\n");
    } // This will be turned off.
    */
    // Better solution?

    std::vector<double> og;
    arma::vec scaled;
//    std::vector<double> scaled;
    unsigned int j;
    arma::vec tmpres;
    double pmean, pvar;

    for(i = 0; i < 8; i++)
    {
        fix_sample(s->at(i));
        scaled.set_size(s->at(i).size());
        //DEBUGING!!!
//        scaled.resize(s->at(i).size());
        for(j = 0; j < s->at(i).size(); j++)
        {
            scaled[j] = s->at(i)[j] / 1024.0;
            // WORKAROUND => port 0 and port 1 do not function appropriately.
        }
//        getFourierFeature<3>(scaled, tmpres);

        pmean = mean(scaled);
        pvar = var(scaled);


        /*for(j = 0; j < 3; j++)
        {
            og.push_back(tmpres[j]);
        }*/
        og.push_back(pmean);
        og.push_back(pvar);
    }

    if(ui->rdRegister->isChecked())
    {
        mr->registerUser(ui->spID->value(), og);
        // showMsg("User Registration Complete.");
    }
    if(ui->rdRecognize->isChecked())
    {
        mt->test(og);
        // showMsg("User Recognized: ID: " + QString::number(d));
    }
}

void MainWindow::regCompl()
{
    showMsg(tr("Registration Complete. "));
}

void MainWindow::tstCompl(unsigned int src)
{
    ucnt[src]++;
    tucnt++;
    showMsg(tr("Recognition Complete. <b>ID: ") + QString::number(src) + "</b>" + "(" + QString::number(ucnt[src]) + "/" + QString::number(tucnt) + ")");
    unsigned int i;
    bool doneflag = false;
    if(tucnt >= 10)
    {
        showMsg(tr("Final Result: "));
        for(i = 0; i < ucnt.size(); i++)
        {
            if((double)ucnt[i] / (double)tucnt >= 0.7)
            {
                doneflag = true;
                showMsg(QString("ID: ") + QString::number(i));
            }
        }
        if(doneflag==false)
        {
            showMsg(tr("Stranger"));
        }
        ucnt.fill(0);
        tucnt = 0;
    }
}

void MainWindow::sNewRecord()
{
    ucnt.fill(0);
    tucnt = 0;
    showMsg(tr("New Record Started."));
}

void MainWindow::valCompl(int tot){
    total_cnt = tot;
    ucnt.resize(tot);
    ucnt.fill(0);
    tucnt = 0;
    showMsg(tr("Validation Complete."));
}

void MainWindow::showProbs(double * data, int num)
{
    for(int i = 0; i < num; i ++)
    {
        showMsg(QString("<b>") + QString::number(i) + ":" + QString::number(data[i]) + "</b>");
    }
}

void MainWindow::chg2nothmod()
{
    ui->gPInfo->show();
    ui->lblID->hide();
    ui->spID->hide();
    ui->btnStartNewRecog->hide();
    showMsg(tr("Changed to Do Nothing Mode."));
}

void MainWindow::chg2recogmod()
{
    ui->gPInfo->hide();
    ui->lblID->hide();
    ui->spID->hide();
    ui->btnStartNewRecog->show();
    showMsg(tr("Changed to Recognition Mode"));
}

void MainWindow::chg2regmod()
{
    ui->gPInfo->hide();
    ui->lblID->show();
    ui->spID->show();
    ui->btnStartNewRecog->hide();
    showMsg(tr("Changed to Registration Mode"));
}

void MainWindow::autoScroll()
{
    QTextCursor cursor = ui->txtMsg->textCursor();
    cursor.movePosition(QTextCursor::End);
    ui->txtMsg->setTextCursor(cursor);
}

void MainWindow::clearMsg()
{
    ui->txtMsg->clear();
}
