#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QLabel>
#include <QLCDNumber>
#include "voltagedata.h"

#include "dataentry.h"
#include "testing.h"
#include "registering.h"
#include "gridcrossvalidating.h"
#define portnum 8

namespace Ui {
class MainWindow;
}

class QSerialPortInfo;
class Buffering;
class VoltageDataCollectingWithWarning;
class VoltageDataCollecting;

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    void showMsg(QString);
    ~MainWindow();
public slots:
    void goListen();
    void stopListen();
    void setCnt();

    void updatePacketStatus(const volt_data* s);

    void evaluateData(std::array<std::vector<unsigned int>, 8>*); // DUMMY

    void updatePacketStat();
    void updateCnt();
    void bufwarn(int);
    void packetwarn(int);
    void packeterr(int);

    void bufstarted();

    void showCurThread(long);

    void regCompl();
    void tstCompl(unsigned int);
    void valCompl(int);

    void showProbs(double*, int);

    void chg2nothmod();
    void chg2recogmod();
    void chg2regmod();

    void sNewRecord();

    void autoScroll();
    void clearMsg();
private:
    Buffering* m_b;
    VoltageDataCollectingWithWarning* m_vdc;

    unsigned int curcnt;

    unsigned int valid_packets;
    unsigned int discarded_packets;
    unsigned int invalid_packets;

    QVector<QLCDNumber*> pvec;

    Ui::MainWindow *ui;

    QList<QSerialPortInfo> info;
    unsigned int total_cnt;
    QVector<unsigned int> ucnt;
    unsigned int tucnt;
// Internal Objects.
    DataEntry* mde;
    Testing* mt;
    Registering* mr;
    GridCrossValidating* mgcv;
};

#endif // MAINWINDOW_H
