#include "registering.h"
#include <cassert>
Registering::Registering(DataEntry& de, QObject *parent) :
    mde(de), QObject(parent)
{
}

void constructProblem(const std::vector<unsigned int>& ids, const std::vector<std::vector<double> >& features, svm_problem* res)
{
    assert(ids.size()==features.size());
    res->l = ids.size();
    res->y = new double[res->l];
    res->x = new svm_node*[res->l];
    unsigned int j;
    for(unsigned i = 0; i < res->l; i++)
    {
        res->y[i] = ids[i];
        res->x[i] = new svm_node[features[i].size()+1];
        for(j = 0; j < features[i].size(); j++)
        {
            res->x[i][j].index = j;
            res->x[i][j].value = features[i][j];
        }
        res->x[i][features[i].size()].index = -1;
    }
}

void deleteProblem(svm_problem* s)
{
    for(unsigned int i = 0; i < s->l; i++)
    {
        delete [] s->x[i];
    }
    delete [] s->y;
}

void Registering::registerUser(unsigned int id, const std::vector<double> & s)
{
    // First, save the feature for future references.
    unsigned int tid = id;
    mde.regFeatureVector(tid, s);

    // Second, get all feature vector and do cross referencing and scaling.
    const std::vector<std::vector<double> >& vecs = mde.getAllFeatureVectors(); // which includes the added vector;
    const std::vector<unsigned int>& ids = mde.getUserList();

    svm_problem* prob = new svm_problem;
    constructProblem(ids, vecs, prob);

    svm_model* resmod;
    svm_parameter defparam = mde.getDefaultParam();
    if(mde.getSVMModel()!=NULL)
    {
        resmod = svm_train(prob, &defparam);
        // Safe Memory Operation?
    }else
    {

        resmod = svm_train(prob, &defparam);
    }
    mde.regSVMModel(resmod);
    emit registerComplete();
    //deleteProblem(prob);
    // Documentation!!!
}
