#include "voltagedatacollecting.h"
#include <QtSerialPort/QSerialPort>
#include <QMessageBox>
#include <QThread>
VoltageDataCollecting::VoltageDataCollecting(const QSerialPortInfo& src, QObject *parent) :
    QObject(parent), open(false)
{
    sp = new QSerialPort(src, this);
    connect(sp, SIGNAL(readyRead()), this, SLOT(readData()));
    connect(sp, SIGNAL(error(QSerialPort::SerialPortError)), this, SLOT(sperror(QSerialPort::SerialPortError)));
}
VoltageDataCollecting::~VoltageDataCollecting()
{
    if(open)
    {
        sp->close();
    }
}

void VoltageDataCollecting::start()
{
    if(open)
    {
        emit error(ERR_ALREADY_OPENED);
        return;
    }
    if(!sp->open(QIODevice::ReadOnly))
    {
        if(sp->error()==2)
            emit error(ERR_NEED_PERMISSION);
        else
            emit error(ERR_UNKNOWN);
        return;
    }
    open = true;
}

void VoltageDataCollecting::stop()
{
    if(!open)
    {
        emit error(ERR_NOT_OPENED);
        return;
    }
    sp->close();
    open = false;
}

void VoltageDataCollecting::readData()
{
    volt_data tmp;
    QByteArray Cur = sp->readAll();
    gqueue.lock();
    total += Cur.length();
    int i;
    for(i = 0; i < Cur.length(); i++)
    {
        queue.push_back(Cur[i]);
    }
    while(queue.length()>=6)
    {
        if((queue[0]^queue[1]^queue[2]^queue[3]^queue[4]^queue[5]^queue[6])==queue[7])
        {
            // Simple Decoding is used to avoid differences between CPU.
            tmp.port = (unsigned char)queue[0];
            tmp.index = ((unsigned char)queue[1] << 24) | ((unsigned char)queue[2] << 16) | ((unsigned char)queue[3] << 8) | (unsigned char)queue[4];
            tmp.voltage = ((unsigned char)queue[5] << 8) | (unsigned char)queue[6];
            emit packetReady(&tmp);
            // Buffering
            queue.pop_front();
            queue.pop_front();
            queue.pop_front();
            queue.pop_front();
            queue.pop_front();
            queue.pop_front();
        }else
        {
            queue.pop_front();
        }
    }
    gqueue.unlock();
}

// ---------------------------I'm a happy seperator.

VoltageDataCollectingWithWarning::VoltageDataCollectingWithWarning(const QSerialPortInfo& src, QObject *parent) :
    QObject(parent), open(false)
{
    sp = new QSerialPort(src, this);
    connect(sp, SIGNAL(readyRead()), this, SLOT(readData()));
}
VoltageDataCollectingWithWarning::~VoltageDataCollectingWithWarning()
{
    if(open)
    {
        sp->close();
    }
}

void VoltageDataCollectingWithWarning::start()
{
    if(open)
    {
        emit error(ERR_ALREADY_OPENED);
        return;
    }
    sp->setBaudRate(QSerialPort::Baud115200);
    if(!sp->open(QIODevice::ReadOnly))
    {
        if(sp->error()==2)
            emit error(ERR_NEED_PERMISSION);
        else
            emit error(ERR_UNKNOWN);
        return;
    }
    open = true;
}

void VoltageDataCollectingWithWarning::stop()
{
    if(!open)
    {
        emit error(ERR_NOT_OPENED);
        return;
    }
    sp->close();
    open = false;
}

void VoltageDataCollectingWithWarning::sperror(QSerialPort::SerialPortError s)
{
    QMessageBox::about(NULL, QString::number(s), QString::number(s));
}

void VoltageDataCollectingWithWarning::readData()
{
    volt_data tmp;
    QByteArray Cur = sp->readAll();
    emit dCur((long)QThread::currentThreadId);
    gqueue.lock();
    int i;
    for(i = 0; i < Cur.length(); i++)
    {
        queue.push_back(Cur[i]);
    }
    while(queue.length()>=8)
    {
        if((queue[0]^queue[1]^queue[2]^queue[3]^queue[4]^queue[5]^queue[6])==queue[7])
        {
            // Simple Decoding is used to avoid differences between CPU.
            tmp.port = (unsigned char)queue[0];
            tmp.index = ((unsigned char)queue[1] << 24) | ((unsigned char)queue[2] << 16) | ((unsigned char)queue[3] << 8) | (unsigned char)queue[4];
            tmp.voltage = ((unsigned char)queue[5] << 8) | (unsigned char)queue[6];
/*            if(tmp.port==0||tmp.port==1)
            {
                tmp.voltage = 281;
            }*/
            if(tmp.port < 8 && tmp.voltage < 1024 && tmp.voltage > 205)
            {
                emit packetReady(&tmp);
            }else
            {
                emit warn(WARN_INVALID_PACKET);
            }
            // Buffering
            queue.pop_front();
            queue.pop_front();
            queue.pop_front();
            queue.pop_front();
            queue.pop_front();
            queue.pop_front();
            queue.pop_front();
            queue.pop_front();
        }else
        {
            warn(WARN_PACKET_CORRUPTED);
            queue.pop_front();
        }
    }
    gqueue.unlock();
}
