#ifndef VOLTAGEDATACOLLECTING_H
#define VOLTAGEDATACOLLECTING_H

#include <QObject>
#include <QMutex>
#include <QtSerialPort/QSerialPort>
#include <QtSerialPort/QSerialPortInfo>
#include "voltagedata.h"

#define ERR_NEED_PERMISSION 1
#define ERR_ALREADY_OPENED 2
#define ERR_NOT_OPENED 3
#define ERR_UNKNOWN 4
#define WARN_PACKET_CORRUPTED 1
#define WARN_INVALID_PACKET 2
#define WARN_INVALID_PACKET_COLLECT 3
#define portnum 8

class QSerialPort;
class QSerialPortInfo;
class VoltageDataCollecting : public QObject
{
    Q_OBJECT
public:
    explicit VoltageDataCollecting(const QSerialPortInfo&, QObject *parent = 0);
    ~VoltageDataCollecting();
    QSerialPort* getSerialPort() { return sp; }
signals:
    void error(int);
    void packetReady(const volt_data*);
public slots:
    void start();
    void stop();
    void readData(); // This is reserved for internal use. Do not connect to it.
private:
    QSerialPort* sp;
    QList<char> queue;
    bool open;
    QMutex gqueue; // This might
    int total;
    int discarded;
};

class VoltageDataCollectingWithWarning : public QObject
{
    Q_OBJECT
public:
    explicit VoltageDataCollectingWithWarning(const QSerialPortInfo&, QObject *parent = 0);
    ~VoltageDataCollectingWithWarning();
    QSerialPort* getSerialPort() { return sp; }
signals:
    void error(int);
    void packetReady(const volt_data*);
    void warn(int);

    void dCur(long);
public slots:
    void start();
    void stop();
    void readData(); // This is reserved for internal use. Do not connect to it.
    void sperror(QSerialPort::SerialPortError);
private:
    QSerialPort* sp;
    QList<char> queue;
    bool open;
    QMutex gqueue; // This might
};

#endif // VOLTAGEDATACOLLECTING_H
